datasets = {
    "tacofoundation/sen2naipv2-unet": [
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-unet.0000.part.taco",
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-unet.0001.part.taco",
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-unet.0002.part.taco",
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-unet.0003.part.taco",
    ],
    "tacofoundation/sen2naipv2-histmatch": [
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-histmatch.0000.part.taco",
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-histmatch.0001.part.taco",
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-histmatch.0002.part.taco",
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-histmatch.0003.part.taco",
    ],
    "tacofoundation/sen2naipv2-crosssensor": [
        "https://huggingface.co/datasets/tacofoundation/SEN2NAIPv2/resolve/main/sen2naipv2-crosssensor.taco",
    ],
}
